import logging
import sys


def setup_logger(logger: logging.Logger) -> None:
    fmt = '%(asctime)s [%(levelname)s] %(message)s'
    datefmt = '%Y-%m-%d %H:%M:%S'
    level = logging.INFO

    formatter = logging.Formatter(fmt=fmt, datefmt=datefmt)
    sh = logging.StreamHandler(sys.stderr)
    sh.setFormatter(formatter)

    logger.setLevel(level)
    logger.addHandler(sh)



import logging
import shlex
import subprocess

logger = logging.getLogger('statusinfo')


def run_command(cmd):
    args = shlex.split(cmd)
    p = subprocess.Popen(args,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.DEVNULL)
    p.wait(timeout=60)

    stdout, _ = p.communicate()
    stdout = stdout.decode(encoding="UTF-8")
    stdout = stdout.strip()

    return stdout



import logging
import sys

from .arguments import parse_arguments
from .exception import StatusInfoArgumentException
from .gatherer import METHOD_CHOICES
from .logging import setup_logger

logger = logging.getLogger('statusinfo')


def main():
    setup_logger(logger)

    try:
        args = parse_arguments()
    except StatusInfoArgumentException as e:
        msg = 'Failed to parse arguments: ' + str(e)
        logger.error(msg)
        exit(1)

    if args.debug:
        logger.setLevel(logging.DEBUG)
        msg = 'Application is running in debug mode.'
        logger.debug(msg)
    elif args.quiet:
        logger.setLevel(logging.WARNING)

    logger.info('Starting StatusInfo.')

    choices = METHOD_CHOICES

    if args.space:
        space = ' ' * 3
    else:
        space = ''

    widget = choices[args.widget]
    icon, data = widget(args.args)
    out = '{icon}  {data}{space}'.format(icon=icon, data=data, space=space)
    sys.stdout.write(out)



import logging
import re

from functools import wraps
from pathlib import Path

from .exception import StatusInfoException
from .util import run_command

logger = logging.getLogger('statusinfo')


# https://stackoverflow.com/a/30904603
def gather(icon):
    def _gather(fn):
        @wraps(fn)
        def with_icon(*args, **kwargs):
            try:
                data = fn(*args, **kwargs)
            except Exception as e:
                data = '?'
                msg = 'Exception while gathering: ' + str(e)
                logger.error(msg)

            return icon, data
        return with_icon
    return _gather


class Gatherer:
    SEPARATOR = '･'

    ICON_BACKLIGHT = ''
    ICON_GIT = ''
    ICON_MAIL = ''
    ICON_MEMORY = ''
    ICON_NEWS = ''
    ICON_SYNCHRONIZATION = ''
    ICON_TEMPERATURE = ''

    @classmethod
    @gather(ICON_BACKLIGHT)
    def gather_backlight(cls, args):
        cmd = 'brightnessctl --machine-readable get'
        stdout = run_command(cmd)
        current = int(stdout)

        cmd = 'brightnessctl --machine-readable max'
        stdout = run_command(cmd)
        maximum = int(stdout)

        percent = int((current / maximum) * 100)
        data = '{:02}%'.format(percent)

        return data

    @classmethod
    @gather(ICON_GIT)
    def gather_git(cls, args):
        if len(args) != 1:
            msg = 'Please specify exactly one source file.'
            raise StatusInfoException(msg)

        filename = args[0]

        with open(filename) as f:
            statistics = f.read()

        values = frozenset(map(int, statistics.split(',')))

        if len(values) != 3:
            msg = 'Source file is corrupt.'
            raise StatusInfoException(msg)

        data = cls.SEPARATOR.join(map(str, values))

        return data

    @classmethod
    @gather(ICON_MAIL)
    def gather_mail(cls, args):
        counts = list()
        args.sort()

        for mailbox in args:
            cmd = 'notmuch count tag:{} and tag:unread'
            cmd = cmd.format(mailbox)
            stdout = run_command(cmd)
            count = int(stdout)
            counts.append(str(count))

        data = cls.SEPARATOR.join(counts)

        return data

    @classmethod
    @gather(ICON_MEMORY)
    def gather_memory(cls, args):
        filename = '/proc/meminfo'

        with open(filename) as f:
            meminfo = f.read()

        pattern = r'^MemTotal: \s+(\d+) kB'
        match_total = re.search(pattern, meminfo)

        pattern = r'MemAvailable: \s+(\d+) kB'
        match_free = re.search(pattern, meminfo)

        if match_total is None or match_free is None:
            raise Exception('Could not parse meminfo.')

        total = int(match_total.group(1))
        free = int(match_free.group(1))
        percent = (free / total) * 100
        percent = 100 - int(percent)
        data = '{:02}%'.format(percent)

        return data

    @classmethod
    @gather(ICON_NEWS)
    def gather_news(cls, args):
        path = '~/news/.count'
        path = Path(path)
        path = path.expanduser()

        with open(path) as f:
            data = f.read()

        data = data.strip()

        return data

    @classmethod
    @gather(ICON_SYNCHRONIZATION)
    def gather_synchronization(cls, args):
        path = '~/.local/share'
        path = Path(path)
        path = path.expanduser()

        pattern = 'sync-*.lock'
        data = path.glob(pattern)
        data = len(list(data))

        return data

    @classmethod
    @gather(ICON_TEMPERATURE)
    def gather_temperature(cls, args):
        path = '/sys/devices/platform/coretemp.0/hwmon'
        path = Path(path)
        pattern = '*/temp*_input'
        files = path.glob(pattern)
        temperatures = list()

        for f in files:
            with open(f) as f:
                temperature = f.read()

            temperature = temperature.strip()
            temperature = int(temperature)
            temperatures.append(temperature)

        data = max(temperatures) // 1000
        data = '{:02}°'.format(data)

        return data


METHOD_CHOICES = {
    'backlight': Gatherer.gather_backlight,
    'git': Gatherer.gather_git,
    'mail': Gatherer.gather_mail,
    'memory': Gatherer.gather_memory,
    'news': Gatherer.gather_news,
    'synchronization': Gatherer.gather_synchronization,
    'temperature': Gatherer.gather_temperature,
}



import logging

from argparse import ArgumentParser

from .exception import StatusInfoArgumentException
from .gatherer import METHOD_CHOICES

logger = logging.getLogger('statusinfo')


def parse_arguments():
    method_choices = METHOD_CHOICES.keys()

    parser = ArgumentParser(
        prog='StatusInfo',
        description='A tool for gathering status information.'
    )
    parser.add_argument('widget', type=str, choices=method_choices,
                        help='Widget to display.')
    parser.add_argument('-v', '--debug', action='store_true',
                        help='Print debug information.')
    parser.add_argument('-q', '--quiet', action='store_true',
                        help='Print errors and warnings only.')
    parser.add_argument('-s', '--space', action='store_true',
                        help='Whether to add space to the right.')
    parser.add_argument('args', nargs='*',
                        help='Additional arguments for the widget.')

    args = parser.parse_args()

    if args.debug and args.quiet:
        msg = 'Cannot be quiet in debug mode.'
        raise StatusInfoArgumentException(msg)

    return args



#!/usr/bin/env python3

from setuptools import setup, find_packages

setup(
    packages=find_packages(exclude=['tests.*', 'tests']),
    use_scm_version={
        'write_to': 'statusinfo/version.py'
    },
    entry_points={
        'console_scripts': ['statusinfo = statusinfo.statusinfo:main']
    },
)
